/******************************************************************************
 * menuman.cpp
 * Drop-down menu manager.
 *-----------------------------------------------------------------------------
 * Copyright (C) 2015 Osman Celimli
 * For conditions of distribution and use, see copyright notice in playtone.cpp
 ******************************************************************************/
#include <Windows.h>
#include <stdint.h>
#include <tchar.h>

#include "../WinTone/wintone.hpp"
#include "toneman.hpp"
#include "menuman.hpp"
#include "resource.h"

/******************************************************************************
 * !!!!----             FILE OPEN AND CLOSE MANAGEMENT               ----!!!!
 ******************************************************************************/
const TCHAR szFileFilter_SMP[] = _T("Sample Pack (*.smp)\0*.SMP\0All Files (*.*)\0*.*\0");
const TCHAR szFileFilter_INS[] = _T("Instrument Pack (*.ins)\0*.INS\0All Files (*.*)\0*.*\0");
const TCHAR szFileFilter_MUS[] = _T("Music Track (*.mus)\0*.MUS\0All Files (*.*)\0*.*\0");
const TCHAR szFileFilter_SFX[] = _T("Sound Effect (*.sfx)\0*.SFX\0All Files (*.*)\0*.*\0");
const TCHAR szFileFilter_WAV[] = _T("Sounds (*.wav)\0*.WAV\0All Files (*.*)\0*.*\0");
const TCHAR szFileFilter_ALL[] = _T("All Files (*.*)\0*.*\0");
const TCHAR szFileExt_WAV[] = _T("wav");

const TCHAR* apszFileFilters[] = {
	szFileFilter_SMP, szFileFilter_INS, szFileFilter_MUS, szFileFilter_SFX,
	szFileFilter_ALL, NULL
};


/* BOOL MenuMan::GetFile(FileType_t eMode)
 * Attempt to open a file whose type is specified by eMode using the windows
 * file open dialog, storing its name and location in mszFileName.
 * Returns true if the user selected a valid file.
 *----------------------------------------------------------------------------*/
BOOL MenuMan::GetFile(FileType_t eMode)
{
	mszFileName[0] = _T('\0');

	if(eMode >= eFILETYPE_FOOTER)
		return FALSE;

	mOpenParams.lpstrFilter = apszFileFilters[eMode];
	return GetOpenFileName(&mOpenParams);
}

/* BOOL MenuMan::SetupLog(void)
 * Prompt the user to select a *.WAV file to save to, caching its name and
 * location in mszSaveName. Returns true if a valid target was chosen.
 *----------------------------------------------------------------------------*/
BOOL MenuMan::SetupLog(void)
{
	mszSaveName[0] = _T('\0');

	return GetSaveFileName(&mSaveParams);
}



/******************************************************************************
 * !!!!----               CONSTRUCTOR AND DESTRUCTOR                 ----!!!!
 ******************************************************************************/
MenuMan::MenuMan(HWND hParent, ToneMan *pToneMan)
{
	mhParent = hParent;
	mpToneMan = pToneMan;

	mhMain = GetMenu(mhParent);
	mhSub_File = GetSubMenu(mhMain, MFILE_SLOT);
	mhSub_Play = GetSubMenu(mhMain, MPLAYBACK_SLOT);

	mOpenParams.lStructSize = sizeof(OPENFILENAME);
	mOpenParams.hwndOwner = mhParent;
	mOpenParams.hInstance = NULL;
	mOpenParams.lpstrFilter = szFileFilter_ALL;
	mOpenParams.lpstrCustomFilter = NULL;
	mOpenParams.nMaxCustFilter = 0;
	mOpenParams.nFilterIndex = 0;
	mOpenParams.lpstrFile = mszFileName;
	mOpenParams.nMaxFile = MENUMAN_BUFFER_LEN;
	mOpenParams.lpstrFileTitle = NULL;
	mOpenParams.nMaxFileTitle = 0;
	mOpenParams.lpstrInitialDir = NULL;
	mOpenParams.lpstrTitle = NULL;
	mOpenParams.Flags = (OFN_FILEMUSTEXIST | OFN_PATHMUSTEXIST | OFN_HIDEREADONLY);
	mOpenParams.nFileOffset = 0;
	mOpenParams.nFileExtension = 0;
	mOpenParams.lpstrDefExt = NULL;
	mOpenParams.lCustData = 0;
	mOpenParams.lpfnHook = NULL;
	mOpenParams.lpTemplateName = NULL;

	mSaveParams.lStructSize = sizeof(OPENFILENAME);
	mSaveParams.hwndOwner = mhParent;
	mSaveParams.hInstance = NULL;
	mSaveParams.lpstrFilter = szFileFilter_WAV;
	mSaveParams.lpstrCustomFilter = NULL;
	mSaveParams.nMaxCustFilter = 0;
	mSaveParams.nFilterIndex = 0;
	mSaveParams.lpstrFile = mszSaveName;
	mSaveParams.nMaxFile = MENUMAN_BUFFER_LEN;
	mSaveParams.lpstrFileTitle = NULL;
	mSaveParams.nMaxFileTitle = 0;
	mSaveParams.lpstrInitialDir = NULL;
	mSaveParams.lpstrTitle = NULL;
	mSaveParams.Flags = (OFN_PATHMUSTEXIST | OFN_OVERWRITEPROMPT);
	mSaveParams.nFileOffset = 0;
	mSaveParams.nFileExtension = 0;
	mSaveParams.lpstrDefExt = szFileExt_WAV;
	mSaveParams.lCustData = 0;
	mSaveParams.lpfnHook = NULL;
	mSaveParams.lpTemplateName = NULL;

	mszFileName[0] = _T('\0');
}

MenuMan::~MenuMan(void)
{
	
}




/******************************************************************************
 * !!!!----                      ABOUT DIALOG                        ----!!!!
 ******************************************************************************/
/* BOOL CALLBACK AboutDlgProc(HWND hWnd, UINT uMsg,
 *                            WPARAM wParam, LPARAM lParam)
 * Callback for the "About" dialog...
 *----------------------------------------------------------------------------*/
BOOL CALLBACK AboutDlgProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg)
	{
		/**
		 * ---- WM_INITDIALOG ----
		 * Received once the dialog is ready and all the controls have
		 * been created, used to set up anything akin to WM_CREATE's
		 * traditional use in normal windows.
		 */
		case WM_INITDIALOG:
			break;

		/**
		 * ---- WM_CLOSE ----
		 */
		case WM_CLOSE:
			EndDialog(hWnd, IDOK);
			break;

		/**
		 * ---- WM_COMMAND ----
		 * Used for button presses in this particular dialog,
		 * of which there's only one: "OK"
		 */
		case WM_COMMAND:
			switch(LOWORD(wParam))
			{
				case IDOK:
					EndDialog(hWnd, IDOK);
					break;
			}

			break;

		/**
		 * Who knows?
		 * However, we -do- know not to use DefWindowProc here,
		 * it's absolutely not for dialogs.
		 */
		default:
			return FALSE;
	}

	return TRUE;
}


/******************************************************************************
 * !!!!----                 MENU WM_COMMAND HANDLER                  ----!!!!
 ******************************************************************************/
/* LRESULT MenuMan::CmdProc(HWND hWnd, UINT uMsg,
 *                          WPARAM wParam, LPARAM lParam)
 * WM_COMMAND handler for the PlayTone window callback.
 *----------------------------------------------------------------------------*/
LRESULT MenuMan::CmdProc(HWND hWnd, UINT uMsg,
                         WPARAM wParam, LPARAM lParam)
{
	HRESULT hRes = E_FAIL;
	INT iStatus,iRetry;

	switch(LOWORD(wParam))
	{
		/**
		 * ---- FILE ----
		 *-----------------------------------*/
		case IDM_FILE_OPEN_SAMPLES:
			if(GetFile(eFILETYPE_SMP))
			{
				mpToneMan->LoadSamples(mszFileName);
			}
			break;

		case IDM_FILE_OPEN_INSTRUMENTS:
			if(GetFile(eFILETYPE_INS))
			{
				mpToneMan->LoadInstruments(mszFileName);
			}
			break;

		case IDM_FILE_OPEN_MUSIC:
			if(GetFile(eFILETYPE_MUS))
			{
				mpToneMan->LoadMusic(mszFileName);
			}
			break;

		case IDM_FILE_OPEN_SFX:
			if(GetFile(eFILETYPE_SFX))
			{
				mpToneMan->LoadSfx(mszFileName);
			}
			break;

		case IDM_FILE_SAVE_WAVE:
			if(SetupLog())
			{
				if(E_FAIL == mpToneMan->StartLog(mszSaveName))
				{
					iStatus = LoadString(GetModuleHandle(NULL),
						IDS_ERR_WAVESTART, mszBuffer, MENUMAN_BUFFER_LEN);

					if(iStatus > 0)
					{
						MessageBox(hWnd, mszBuffer, NULL,
							(MB_OK | MB_ICONEXCLAMATION));
					}
				}
			}
			break;

		case IDM_FILE_STOP_WAVE:
			mpToneMan->StopLog();
			break;

		case IDM_FILE_EXIT:
			PostMessage(hWnd, WM_CLOSE, 0, 0);
			break;


		/**
		 * ---- PLAYBACK ----
		 *-----------------------------------*/
		case IDM_PLAYBACK_PAUSE:
			mpToneMan->Pause();
			break;

		case IDM_PLAYBACK_RESUME:
			mpToneMan->Resume();
			break;

		case IDM_PLAYBACK_STOP_ALL:
			mpToneMan->StopAll();
			break;

		case IDM_PLAYBACK_MUSIC_PLAY:
			mpToneMan->PlayMusic();
			break;

		case IDM_PLAYBACK_MUSIC_STOP:
			mpToneMan->StopMusic();
			break;

		/**
		 * !!!!---- WOAH, HEY ----!!!!
		 * These are now caught by the sound effects manager in
		 * StatMan.cpp which allows panning and priority control.
		 *
		case IDM_PLAYBACK_SFX_PLAY:
			mpToneMan->PlaySfx(127, 127, 127);
			break;

		case IDM_PLAYBACK_SFX_STOP:
			mpToneMan->StopSfx();
			break;
		 *
		 ***/


		/**
		 * ---- ABOUT ----
		 *-----------------------------------*/
		case IDM_HELP_ABOUT:
			DialogBox(GetModuleHandle(NULL), MAKEINTRESOURCE(IDD_ABOUT),
				hWnd, AboutDlgProc);
			break;
	}

	/**
	 * If both the sample and instrument packages have been loaded and
	 * the driver is currently not running, try to start it.
	 */
	if(!(mpToneMan->mbDriverRunning) && mpToneMan->mbSamplesReady
		&& mpToneMan->mbInstrumentsReady)
	{
		do{
			iRetry = 0;
			hRes = mpToneMan->StartDriver();

			if(FAILED(hRes))
			{
				iStatus = LoadString(GetModuleHandle(NULL), IDS_ERR_WTSTART,
					mszFileName, MENUMAN_BUFFER_LEN);

				if(iStatus > 0)
				{
					iStatus = MessageBox(hWnd, mszFileName, NULL,
						(MB_RETRYCANCEL | MB_ICONEXCLAMATION));
					iRetry = (IDRETRY == iStatus)
						? -1 : 0;
				}
			}
		}while(iRetry);
	}

	RefreshAll();
	DrawMenuBar(hWnd);
	return 0;
}


/******************************************************************************
 * !!!!----                         REFRESH                          ----!!!!
 ******************************************************************************/
/* void MenuMan::RefreshAll(void)
 * Update the entire menu tree based upon the current driver state.
 *----------------------------------------------------------------------------*/
void MenuMan::RefreshAll(void)
{
	EnableMenuItem(mhSub_File, MFILE_OPEN_MUSIC_SUBSLOT,
		((mpToneMan->mbSamplesReady && mpToneMan->mbInstrumentsReady)
			? (MF_BYPOSITION | MF_ENABLED)
			: (MF_BYPOSITION | MF_GRAYED)));
	EnableMenuItem(mhSub_File, MFILE_OPEN_SFX_SUBSLOT,
		((mpToneMan->mbSamplesReady && mpToneMan->mbInstrumentsReady)
			? (MF_BYPOSITION | MF_ENABLED)
			: (MF_BYPOSITION | MF_GRAYED)));
	EnableMenuItem(mhSub_File, MFILE_START_LOG_SUBSLOT,
		((mpToneMan->mbDriverRunning && !(mpToneMan->mbLogging))
			? (MF_BYPOSITION | MF_ENABLED)
			: (MF_BYPOSITION | MF_GRAYED)));
	EnableMenuItem(mhSub_File, MFILE_STOP_LOG_SUBSLOT,
		((mpToneMan->mbDriverRunning && mpToneMan->mbLogging)
			? (MF_BYPOSITION | MF_ENABLED)
			: (MF_BYPOSITION | MF_GRAYED)));

	EnableMenuItem(mhMain, MPLAYBACK_SLOT,
		(mpToneMan->mbDriverRunning
			? (MF_BYPOSITION | MF_ENABLED)
			: (MF_BYPOSITION | MF_GRAYED)));
	EnableMenuItem(mhSub_Play, MPLAYBACK_PLAYMUS_SUBSLOT,
		(mpToneMan->mbMusicReady
			? (MF_BYPOSITION | MF_ENABLED)
			: (MF_BYPOSITION | MF_GRAYED)));
	EnableMenuItem(mhSub_Play, MPLAYBACK_STOPMUS_SUBSLOT,
		(mpToneMan->mbMusicReady
			? (MF_BYPOSITION | MF_ENABLED)
			: (MF_BYPOSITION | MF_GRAYED)));
	EnableMenuItem(mhSub_Play, MPLAYBACK_PLAYSFX_SUBSLOT,
		(mpToneMan->mbSfxReady
			? (MF_BYPOSITION | MF_ENABLED)
			: (MF_BYPOSITION | MF_GRAYED)));
	EnableMenuItem(mhSub_Play, MPLAYBACK_STOPSFX_SUBSLOT,
		(mpToneMan->mbSfxReady
			? (MF_BYPOSITION | MF_ENABLED)
			: (MF_BYPOSITION | MF_GRAYED)));
}
